package org.gcube.application.geoportal.clients;

import com.fasterxml.jackson.databind.ser.std.StdKeySerializers;
import org.gcube.application.geoportal.client.DefaultProfiledDocumentsClient;
import org.gcube.application.geoportal.common.model.document.ProfiledDocument;
import org.gcube.application.geoportal.common.model.rest.Configuration;
import org.gcube.application.geoportal.common.model.rest.QueryRequest;
import org.gcube.application.geoportal.common.rest.ProfiledDocumentsI;
import org.gcube.application.geoportal.common.utils.tests.GCubeTest;
import org.junit.Test;

import java.rmi.RemoteException;
import java.util.concurrent.atomic.AtomicLong;

import static junit.framework.TestCase.assertTrue;
import static org.gcube.application.geoportal.client.GeoportalAbstractPlugin.*;
import static org.junit.Assume.assumeTrue;

public class ProfiledDocumentsTest<M extends ProfiledDocument,C extends ProfiledDocumentsI<M>> extends BasicVreTests{

    protected String getProfileID(){return "profiledConcessioni";}

    protected C getClient(){
        return (C) profiledModel(getProfileID(),ProfiledDocument.class,DefaultProfiledDocumentsClient.class).build();
    }


    @Test
    public void getConfiguration() throws Exception {
        assumeTrue(GCubeTest.isTestInfrastructureEnabled());
        ProfiledDocumentsI<M> client = (ProfiledDocumentsI<M>) getClient();
        Configuration config=client.getConfiguration();
        System.out.println("Configuration is "+ config);
    }


    @Test
    public void list() throws Exception {
        assumeTrue(GCubeTest.isTestInfrastructureEnabled());
        ProfiledDocumentsI<M> client = (ProfiledDocumentsI<M>) getClient();
        AtomicLong counter=new AtomicLong(0);
        client.query(new QueryRequest()).forEachRemaining( M -> counter.incrementAndGet());
        System.out.println("Found "+counter.get()+" elements");
        System.out.println("Getting JSON ");
        System.out.println(client.queryForJSON(new QueryRequest()));
    }

    @Test
    public void scanStatusByID() throws Exception {
        assumeTrue(GCubeTest.isTestInfrastructureEnabled());
        ProfiledDocumentsI<M> client = (ProfiledDocumentsI<M>) getClient();
        AtomicLong counter=new AtomicLong(0);
        client.query(new QueryRequest()).forEachRemaining( m -> {
            System.out.print(counter.incrementAndGet()+ ", ID : "+m.getId());
            try {
                M proj=client.getById(m.getId());
                System.out.println("... OK.. STATUS : "+proj.getLifecycleInformation());
            } catch (RemoteException e) {
                System.err.println(" Error with "+m.getId());
                System.err.println(e);
                System.out.println("... ERRR !!!!");
            }
        });
    }




}
