package org.gcube.application.geoportal.service.engine.mongo;

import com.fasterxml.jackson.core.JsonProcessingException;
import org.bson.Document;
import org.gcube.application.cms.plugins.faults.EventException;
import org.gcube.application.cms.plugins.faults.StepException;
import org.gcube.application.geoportal.common.faults.StorageException;
import org.gcube.application.geoportal.common.model.rest.Configuration;
import org.gcube.application.geoportal.common.model.rest.QueryRequest;
import org.gcube.application.geoportal.common.model.rest.RegisterFileSetRequest;
import org.gcube.application.geoportal.common.model.rest.ConfigurationException;
import org.gcube.application.geoportal.service.model.internal.faults.DeletionException;
import org.gcube.common.storagehub.model.exceptions.StorageHubException;

import java.io.IOException;

public interface MongoManagerI<T> {


    // create

    public T registerNew(Document toRegister) throws IOException, StepException, EventException;

    // update
    public T update(String id,Document toSetDocument) throws IOException, StepException, EventException;

    // delete

    public void delete(String id,boolean force) throws DeletionException;

    // get By ID

    public T getByID(String id) throws IOException;

    // query

    public Iterable<Document> query(QueryRequest request);
    public Iterable<T> filter(QueryRequest request);

    public T performStep(String id, String step, Document options) throws IOException, StepException;

    public T registerFileSet(String id, RegisterFileSetRequest request) throws ConfigurationException, StorageHubException, StorageException, StepException, JsonProcessingException, DeletionException, EventException;
    public T deleteFileSet(String id, String destination, Boolean force) throws ConfigurationException, StorageHubException, StorageException, StepException, JsonProcessingException, DeletionException, EventException;

    public Configuration getConfiguration()throws ConfigurationException;
}
